@@ -1,5 +1,6 @@
 import typing as t
 from . import objects as obj
+import networkx as nx  # type: ignore
 
 
 def make_sequence_create(sequence: obj.Sequence) -> str:
@@ -34,6 +35,8 @@ def make_enum_create(enum: obj.Enum) -> str:
         ", ".join("'%s'" % e for e in enum["elements"])
     )
 
+def make_constraint(constraint: obj.Constraint) -> str:
+    return "CONSTRAINT %s %s" % constraint
 
 def make_table_create(table: obj.Table) -> str:
     column_statements = []
@@ -41,12 +44,20 @@ def make_table_create(table: obj.Table) -> str:
         column = get_column(table, col_name)
         column_str = make_column(column)
         column_statements.append(column_str);
-    return "CREATE {}TABLE {} ({})".format(
+    rv = "CREATE {}TABLE {} ({}".format(
         "UNLOGGED" if table["persistence"] == "u" else "",
         table["name"],
         ", ".join(column_statements)
     )
-
+    constraints = [
+        make_constraint(get_constraint(table, c))
+        for c in table["constraints"]
+    ]
+    if constraints:
+        rv = "{}, {})".format(rv, ", ".join(constraints))
+    else:
+        rv = rv + ")"
+    return rv
 
 def make_column(column: obj.Column) -> str:
     name, type, default, notnull = column
@@ -75,6 +86,8 @@ def get_obj_id(obj: obj.DBObject) -> str:
         return "%s.%s" % (obj["schema"], obj["signature"])
     if obj["obj_type"] == "trigger":
         return "%s.%s" % (obj["schema"], obj["name"])
+    if obj["obj_type"] == "dependency":
+        return "%s.%s" % (obj["identity"], obj["dependency_identity"])
     raise ValueError("Invalid obj: %s" % obj)
 
 
@@ -101,3 +114,17 @@ def format_statement(statement: str) -> str:
     if not statement.endswith(";"):
         statement = statement + ";"
     return statement
+
+
+def topo_sort(graph: "nx.DiGraph", items: t.Dict[str, t.List[str]]) -> t.List[str]:
+    rv = []
+    for dep in nx.topological_sort(graph):
+        try:
+            rv.extend(items.pop(dep))
+        except KeyError:
+            pass
+
+    for _, statements in items.items():
+        rv.extend(statements)
+
+    return rv
